#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/3/31 22:33   satan      1.0         None
"""
from JCoder_redis import Redis

class Float(Redis):
    def __init__(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True):
        """
        init
        """
        super().__init__(host,port,default_db,is_decode)

    def __getitem__(self, item: str) -> float:
        """
        获取值
        :param item:
        :return:
        """
        return self._redis.get(item)

    def __setitem__(self, key: str, value: tuple) -> None:
        """
        设置值
        :param key:
        :param value:
        :return:
        """
        self._redis.set(key,value) if type(value)==float else self._redis.set(name=key,value=value[0],ex=value[1])

    def __iadd__(self, other: tuple):
        """
        自增
        :param other:
        :return:
        """
        self._redis.incrbyfloat(other[0],other[1])
        return self


#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/3/31 23:01   satan      1.0         None
"""
from JCoder_redis import Redis

class Set(Redis):
    def __init__(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True):
        """
        init
        """
        super().__init__(host,port,default_db,is_decode)

    def __getitem__(self, item: str) -> set:
        """

        :param item:
        :return:
        """
        return self._redis.smembers(item)

    def __setitem__(self, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        self._redis.delete(key)
        # print(type(value))
        if type(value)==tuple:
            self._redis.sadd(key,*value[0])
            self._redis.expire(key,value[1])
        else:
            self._redis.sadd(key, *value)

    def __add__(self, other:tuple) -> set :
        """

        :param other:
        :return:
        """
        return self._redis.sunion(*other)

    def ergodic(self, key,match=None,count=None):
        """

        :param key:
        :param match:
        :param count:
        :return:
        """
        return self._redis.sscan_iter(key,match,count)

    def __iand__(self, other):
        """
        other[0]=other[0]并other[1]
        :param other:
        :return:
        """
        self._redis.sunionstore(other[0],other[0],other[1])
        return self

    def __sub__(self, other) -> set :
        """
        多集合取交
        :param other:
        :return:
        """
        return self._redis.sdiff(*other)

    def __isub__(self, other):
        """

        :param other:
        :return:
        """
        self._redis.sdiffstore(other[0],other[0],other[1])
        return self

    def __and__(self, other) -> set :
        """
        多集合取并
        :param other:
        :return:
        """
        return self._redis.sinter(*other)

    def __iand__(self, other):
        """

        :param other:
        :return:
        """
        self._redis.sinterstore(other[0],other[0],other[1])
        return self

    def exist(self,keys: tuple) -> bool :
        """

        :param keys:
        :return:
        """
        if type(keys)!=tuple:
            return super().exists(keys)
        return self._redis.sismember(keys[0],keys[1])

    def __rshift__(self,other):
        """
        将成员other[2]从other[0]集合中移动到other[1]集合
        :param other:
        :return:
        """
        self._redis.smove(other[0],other[1],other[2])

    def pop(self,key: str):
        """
        随机删除并返回
        :param key:
        :return:
        """
        return self._redis.spop(key)

    def remove(self,key: str,val):
        """
        删除指定的值
        :param key:
        :param val:
        :return:
        """
        self._redis.srem(key,*val)

    def length(self,key: str) -> int :
        """

        :param key:
        :return:
        """
        return self._redis.scard(key)



from JCoder_redis import Redis


class Ordered_set(Redis):

    def __init__(self, host: str = 'localhost', port: int = 6379, default_db: int = 0, is_decode: bool = True):
        """

        :param host:
        :param port:
        :param default_db:
        :param is_decode:
        """
        super().__init__(host, port, default_db, is_decode)

    def __setitem__(self, key: str, value):
        """

        :param key:
        :param value:
        :return:
        """
        super().delete(key)
        for i in (value[0] if type(value)==tuple else value):
            self._redis.zadd(key, i)
        if type(value)==tuple:
            super().expire(key,value[1])

    def __iadd__(self, other:tuple):
        """

        :param other:
        :return:
        """
        for i in other[1]:
            self._redis.zadd(other[0],i)

    def length(self,key:str) -> int :
        """

        :param key:
        :return:
        """
        return self._redis.zcard(key)

    def __getitem__(self, item:tuple):
        """

        :param item:
        :return:
        """
        return self._redis.zrange(item[0],0,-1,withscores=item[1])

    def get_range(self,name:str,start:int=0,end:int=-1,desc:bool=False,withscores:bool=False,score_cast_func=float):
        """

        :param name:
        :param start:
        :param end:
        :param desc:
        :param withscores:
        :param score_cast_func:
        :return:
        """
        return self._redis.zrange(name=name,start=start,end=end,desc=desc,withscores=withscores,score_cast_func=score_cast_func)

    def get_range_by_score(self,name:str,min,max,start:int=None,num:int=None,desc:bool=False,withscores:bool=False,score_cast_func=float):
        """

        :param name:
        :param min:
        :param max:
        :param start:
        :param num:
        :param desc:
        :param withscores:
        :param score_cast_func:
        :return:
        """
        if desc:
            return self._redis.zrangebyscore(name=name,min=min,max=max,start=start,num=num,withscores=withscores,score_cast_func=score_cast_func)
        return self._redis.zrevrangebyscore(name=name,min=min,max=max,start=start,num=num,withscores=withscores,score_cast_func=score_cast_func)

    def get_range_by_lex(self,name:str,min,max,start:int=None,num:int=None,desc:bool=False,withscores:bool=False):
        """

        :param name:
        :param min:
        :param max:
        :param start:
        :param num:
        :param desc:
        :param withscores:
        :param score_cast_func:
        :return:
        """
        if desc:
            return self._redis.zrangebylex(name=name,min=min,max=max,start=start,num=num,withscores=withscores)
        return self._redis.zrevrangebylex(name=name,min=min,max=max,start=start,num=num,withscores=withscores)

    def scan(self,name:str,cursor:int=0,match=None,count=None,score_cast_func=float):
        """
        获取元素--默认按照分数顺序排序
        :param name:
        :param cursor:
        :param match:
        :param count:
        :param score_cast_func:
        :return:
        """
        return self._redis.zscan(name=name,cursor=cursor,match=match,count=count,score_cast_func=score_cast_func)

    def ergodic(self,name:str,match=None,count=None,score_cast_func=float):
        """

        :param name:
        :param match:
        :param count:
        :param score_cast_func:
        :return:
        """
        return self._redis.zscan_iter(name=name,match=match,count=count,score_cast_func=score_cast_func)

    def count(self,name:str,min,max) -> int:
        """
        获取name对应的有序集合中分数 在 [min,max] 之间的个数
        :param min:
        :param max:
        :return:
        """
        return self._redis.zcount(name=name,min=min,max=max)

    def increase(self,name:str,key:str,amount):
        """

        :param name:
        :param key:
        :param amount:
        :return:
        """
        self._redis.zincrby(name=name,amount=amount,value=key)

    def get_rank(self,name:str,key) -> int:
        """

        :param name:
        :param key:
        :return:
        """
        return self._redis.zrank(name,key)

    def get_score(self,name:str,key):
        """

        :param name:
        :param key:
        :return:
        """
        return self._redis.zscore(name,key)

    def del_by_score(self,name:str,min,max):
        """

        :param name:
        :param min:
        :param max:
        :return:
        """
        self._redis.zremrangebyscore(name=name,min=min,max=max)

    def del_by_rank(self,name:str,min:int,max:int):
        """

        :param name:
        :param min:
        :param max:
        :return:
        """
        self._redis.zremrangebyrank(name=name,min=min,max=max)

    def del_by_lex(self,name:str,min,max):
        """

        :param name:
        :param min:
        :param max:
        :return:
        """
        self._redis.zremrangebylex(name=name,min=min,max=max)

    def remove(self,name:str,*key):
        """

        :param name:
        :param key:
        :return:
        """
        self._redis.zrem(name=name,*key)


from dict import Dict
from int import Int
from float import Float
from set import Set
from list import List
from ordered_set import Ordered_set

def test_int():
    x=Int()
    x['test_int']=10
    x+='test_int',1
    print(x['test_int'])

def test_float():
    x=Float()
    x['test_f']=10.1
    x+='test_f',1.1
    print(x['test_f'])

def test_set():
    x=Set()
    x['test_set1']={1,2,3}
    x['test_set2']={5,6,7}
    x>>('test_set2','test_set1',5)
    print(x['test_set1'])
    print(x['test_set2'])
    # print(x['test_set'])
    # print(x+('test_set','test_set'))

    # for i in x.ergodic('test_set') :
    #     print(type(i))
    #     print(i)
    # for i in x['test_set']:
    #     print(type(i))
def test_list():
    x=List()
    x['list1']=[1,2,3],3
    # x+=('list1',[5,6,7])
    # x.push_front('list1',[1,2,3])
    # x['list2']=[-1,-2,-3]
    # x>>('list1','list2')
    # for i in x.ergodic('list1',0):
    #     print(i)
    print(x['list1'])

def dict_test():
    x=Dict()
    x['dict1']={'1':1,'2':2,'3':3}
    # x['dict1','5']='5'
    # del x['dict1','5']
    # for i in x.ergodic('dict1'):
    #     print(i)
    print(x['dict1'])

# x=Redis()
# x.exists('name')
# dict_test()
x=Ordered_set()
x['test']=[{1:30},{2:20},{3:10}]
print(x['test',True])


#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/4/10 11:01   satan      1.0         None
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="JCoder_redis",
    version="0.0.1",
    author="satan404",
    author_email="11912823@mail.sustech.edu.cn",
    description="This Redis is a Python library based on redis.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/satans404/repackaged_redis_for_JCoder",
    packages=setuptools.find_packages(),
    install_requires = ['redis'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/3/31 22:33   satan      1.0         None
"""
from JCoder_redis import Redis

class List(Redis):
    def __init__(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True):
        """
        init
        """
        super().__init__(host,port,default_db,is_decode)

    def __getitem__(self, item: tuple):
        """
        获取值
        :param item:
        :return:
        """
        if type(item)==tuple:
            return self._redis.lindex(item[0],item[1])
        else:
            return self._redis.lrange(item,0,-1)

    def __setitem__(self, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        if type(key)==str:
            self._redis.delete(key)
            if type(value)==tuple:
                self._redis.rpush(key,*value[0])
                self._redis.expire(key,value[1])
            else:
                self._redis.rpush(key, *value[0])
        else:
            self._redis.lset(key[0],key[1],value)

    def __iadd__(self, other: tuple):
        """

        :param other:
        :return:
        """
        self._redis.rpush(other[0],*other[1])
        return self

    def length(self,key: str) -> int :
        """
        获取长度
        :param key:
        :return:
        """
        return self._redis.llen(key)

    def push_front(self, key,value):
        """
        从前添加
        :param key:
        :param value:
        :return:
        """
        self._redis.lpush(key,*value)
        return self

    def insert_before(self,key,num,value):
        """
        在num前插入value，注意此处的num是指数组中的值，不是下标
        :param key:
        :param num:
        :param value:
        :return:
        """
        self._redis.linsert(key,"before",num,value)

    def insert_after(self,key,num,value):
        """

        :param key:
        :param num:
        :param value:
        :return:
        """
        self._redis.linsert(key,"after",num,value)

    def remove(self,key,index,num):
        """
        删除对应值的元素
        :param key:
        :param index:
        :param num:删除的个数，0：全部，>0从前向后的num个，<0从后向前的num个
        :return:
        """
        self._redis.lrem(key,index,num)

    def pop_front(self,key):
        """
        删除并返回第一个值
        :param key:
        :return:
        """
        return self._redis.lpop(key)

    def pop_back(self,key):
        """
        删除并返回第二个值
        :param key:
        :return:
        """
        return self._redis.rpop(key)

    def remain_range(self,key: str,start: int,end: int):
        """
        删除索引以外的值
        :param key:
        :param start:
        :param end:
        :return:
        """
        return self._redis.lrange(key,start,end)

    def __rshift__(self, other):
        """
        从一个列表取出最右边的元素，同时将其添加至另一个列表的最左边
        :param other:
        :return:
        """
        self._redis.rpoplpush(other[0],other[1])

    def remove_lists(self,keys):
        """
        批量删除list
        :param keys:
        :return:
        """
        self._redis.blpop(*keys)

    def ergodic(self,key: str,_from: int=0,_to: int=-1):
        """
        自定义增量迭代器
        :param key:
        :param _from:
        :param _to:
        :return:
        """
        if _to<0:
            _to=self.length(key)
        for index in range(_from,_to):
            yield self._redis.lindex(key,index)


#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/3/31 22:33   satan      1.0         This Redis is a Python library based on redis.
"""
import redis

class Redis:
    """
    :Description
    This Redis is a Python library based on redis.

    """
    def __init__(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True):
        """
        init connection
        """
        try:
            self._redis=redis.StrictRedis(host=host, port=port, db=default_db, decode_responses=is_decode)
        except BaseException as redis_connect_error:
            raise redis_connect_error

    def expire(self,key: str,num: int):
        """
        设置键值的过期时间
        :param key:键
        :param num:过期时间，单位秒
        :return:
        """
        self._redis.expire(key,num)

    def delete(self,key):
        """
        删除特定键
        :param key:要删除的键
        :return:
        """
        self._redis.delete(*key)

    def exists(self,key: str):
        """
        判断键是否存在
        :param key:要查询的键
        :return:
        """
        return self._redis.exists(key)

    def rename(self,src: str,dst: str):
        """
        重命名
        :param src:原名
        :param dst:新名字
        :return:
        """
        return self._redis.rename(src,dst)

    def reconnect(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True) -> bool:
        """

        :param host:
        :param port:
        :param default_db:
        :param is_decode:
        :return:
        """
        try:
            self._redis.close()
            self._redis=redis.StrictRedis(host=host, port=port, db=default_db, decode_responses=is_decode)
            return True
        except BaseException as redis_connect_error:
            return False

    def close(self):
        """

        :return:
        """
        self._redis.close()


#!/usr/bin/python3
# -*- encoding: utf-8 -*-
"""
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/3/31 22:33   satan      1.0         None
"""
from JCoder_redis import Redis

class Dict(Redis):
    def __init__(self,host :str='localhost',port: int=6379,default_db :int=0,is_decode :bool =True):
        """
        init
        """
        super().__init__(host,port,default_db,is_decode)

    def __getitem__(self, item:tuple):
        """
        获取对象
        :param item:如果item是tuple类型，则返回item[0]->{item[1]:(return)}，否则返回item->(return)
        :return:
        """
        if type(item)==tuple:
            return self._redis.hget(item[0],item[1])
        else:
            return self._redis.hgetall(item)

    def __setitem__(self, key:tuple , value):
        """
        设置对象
        :param key:key[0]主键，key[1]内部键
        :param value:value[0]值，value[1]过期时间
        :return:
        """
        if type(key)==tuple:
            self._redis.hset(key[0], key[1], value)
        else:
            if type(value) == tuple:
                self._redis.hmset(key,value[0])
                self._redis.expire(key,value[1])
            else:
                self._redis.hmset(key, value)

    def get_items(self,key:str, items: list) -> list:
        """
        批量获取对象
        :param key:主键
        :param items:内部键，为数组类型
        :return:
        """
        return self._redis.hmget(key,items)

    def get_keys(self,key: str) -> list:
        """
        得到内部键
        :param key:主键
        :return:
        """
        return self._redis.hkeys(key)

    def get_values(self,key: str) -> list:
        """
        得到内部值
        :param key:主键
        :return:
        """
        return self._redis.hvals(key)

    def length(self,key: str) -> int:
        """
        字典长度
        :param key:主键
        :return:
        """
        return self._redis.hlen(key)

    def exists(self,name:str,key:str) -> bool:
        """
        判断给定键是否存在
        :param name:主键
        :param key:内部键
        :return:
        """
        return self._redis.hexists(name,key)

    def __delitem__(self, key:tuple):
        """
        删除元素
        :param key:key[0]主键,key[1]内部键
        :return:
        """
        self._redis.hdel(key[0],*key[1])

    def __iadd__(self, other: tuple):
        """
        自增，内部键对应的类型必须为int或float
        :param other:other[0]主键,other[1]内部键,other[2]增幅
        :return:
        """
        self._redis.hincrby(other[0],other[1],other[2]) if type(other[2])==int else self._redis.hincrbyfloat(other[0],other[1],other[2])

    def get_range(self,name:str,cursor :int=0,match=None,count: int=None):
        """
        切片
        :param name:
        :param cursor:
        :param match:
        :param count:
        :return:
        """
        return self._redis.hscan(name,cursor,match,count)

    def ergodic(self,name:str,match=None,count=None):
        """
        迭代器遍历
        :param name:
        :param match:
        :param count:
        :return:
        """
        return self._redis.hscan_iter(name,match,count)
            # yield i

